<?php

namespace App\Http\Controllers\Client;

use App\Http\Controllers\Controller;
use App\Http\Requests\ConnectRequest;
use App\Models\Server;
use App\Models\User;
use Illuminate\Support\Facades\Auth;

class ConnectController extends Controller
{
    /**
     *  @OA\Post(
     *    tags={"Clients:Server"},
     *    path="/server/{id}/connect",
     *    summary="Connect to a server",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/Connect"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="403",
     *      description="Forbidden",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="message",
     *          type="string",
     *        ),
     *      )
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="ip",
     *          type="string",
     *          format="ip"
     *        ),
     *        @OA\Property(
     *          property="port",
     *          type="integer"
     *        ),
     *        @OA\Property(
     *          property="pubkey",
     *          type="string",
     *          pattern="^-----BEGIN PUBLIC KEY-----(\n|\r|\r\n)([0-9a-zA-Z\+\/=]{64}(\n|\r|\r\n))*([0-9a-zA-Z\+\/=]{1,63}(\n|\r|\r\n))?-----END PUBLIC KEY-----$"
     *        ),
     *      )
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function connect(ConnectRequest $request, $id)
    {
        $user = User::findOrFail(Auth::guard('client')->id());

        if ($user->server_state == 'connected') {
            if ($user->server_id != null) {
                if ($user->server->online) {
                    return response()->json([
                        'message' => 'Client is already connected',
                    ], 403);
                }
            }

            $user->disconnect();
        }

        $server = Server::findOrFail($id);
        if (!$server->online) {
            return response()->json([
                'message' => 'Server is offline',
            ], 403);
        }

        $user->connecting($server, $request->input('token'));
        return response()->json([
            "ip" => $server->ip,
            "port" => $server->port,
            "pubkey" => $server->pubkey
        ]);
    }
}