<?php

namespace App\Http\Controllers\Shared;

use App\Http\Controllers\Controller;
use App\Http\Requests\CreateGameRequest;
use App\Http\Requests\UpdateGameRequest;
use App\Models\Game;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class GameController extends Controller
{
    /**
     *  @OA\Get(
     *    tags={"Clients:Game"},
     *    path="/game",
     *    summary="Get game list",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="games",
     *          type="array",
     *          @OA\Items(ref="#/components/schemas/Game")
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function index()
    {
        return response()->json([
            "games" => Game::with("users:id")->get()
        ]);
    }

    /**
     *  @OA\Get(
     *    tags={"Clients:Game"},
     *    path="/game/{id}",
     *    summary="Get a game",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="game",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/Game"),
     *            @OA\Schema(
     *              @OA\Property(
     *                property="server",
     *                allOf={
     *                  @OA\Schema(ref="#/components/schemas/Server"),
     *                  @OA\Schema(nullable=true)
     *                }
     *              ),
     *            )
     *          }
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function show($id)
    {
        return response()->json([
            "game" => Game::findOrFail($id)->load("server")
        ]);
    }

    /**
     *  @OA\Get(
     *    tags={"Clients:Game"},
     *    path="/game/{id}/user",
     *    summary="Get users from a game",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="users",
     *          type="array",
     *          @OA\Items(
     *            allOf={
     *              @OA\Schema(ref="#/components/schemas/User"),
     *              @OA\Schema(ref="#/components/schemas/UserGame")
     *            }
     *          )
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */

    public function index_users($id)
    {
        return response()->json([
            "users" => Game::findOrFail($id)->users
        ]);
    }

    /**
     *  @OA\Get(
     *    tags={"Clients:Game"},
     *    path="/game/{id}/user/{user_id}",
     *    summary="Get a user from a game",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Parameter(
     *      name="user_id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="user",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/User"),
     *            @OA\Schema(ref="#/components/schemas/UserGame")
     *          }
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */

    public function show_users($id, $user_id)
    {
        return response()->json([
            "user" => Game::findOrFail($id)->users()->findOrFail($user_id)
        ]);
    }

    /**
     *  @OA\Post(
     *    tags={"Servers:Game"},
     *    path="/game",
     *    summary="Create a game",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/CreateGame"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="403",
     *      ref="#/components/responses/403"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="201",
     *      description="Resource created",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="game",
     *          ref="#/components/schemas/Game",
     *        ),
     *      ),
     *    ),
     *    security={{"server-jwt":{}}},
     *  )
     */
    public function store(CreateGameRequest $request)
    {
        $server = Auth::guard('server')->user();

        DB::beginTransaction();

        $game = Game::create([
            "server_id" => $server->id,
            "mode" => $request->input("mode"),
        ]);

        foreach ($request->input("users") as $u) {
            $user = User::findOrFail($u["id"]);
            if ($user->server_id === $server->id && $user->server_state === 'connected') {
                if ($game->mode == "team")
                    $game->users()->attach($user->id, ['team_id' => $u['team_id']]);
                else
                    $game->users()->attach($user->id, ['team_id' => null]);
            } else {
                DB::rollBack();
                return response()->json([
                    "message" => "Forbidden access to user " . $user->id
                ], 403);
            }
        }

        DB::commit();

        $game->start();

        return response()->json([
            "game" => $game
        ], 201);
    }

    /**
     *  @OA\Put(
     *    tags={"Servers:Game"},
     *    path="/game/{id}",
     *    summary="Update a game",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/UpdateGame"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="403",
     *      ref="#/components/responses/403"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="Resource updated",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="game",
     *          ref="#/components/schemas/Game",
     *        ),
     *      ),
     *    ),
     *    security={{"server-jwt":{}}},
     *  )
     */
    public function update(UpdateGameRequest $request, $id)
    {
        $server = Auth::guard('server')->user();

        $game = Game::findOrFail($id);

        if ($game->server_id !== $server->id)
        {
            abort(404);
        }
        
        if ($game->state === 'playing' && $request->input('state') === 'finished')
        {
            foreach($request->input('users') as $user) {
                if ($game->users()->find($user["id"]) == null) {
                    return response()->json([
                        "message" => "Forbidden access to user " . $user["id"]
                    ], 403);
                }
            }


            $game->end($request->input('users'));
            return response()->json([
                "game" => $game
            ], 200);
        }

        return response()->json([
            "message" => "Invalid state."
        ], 403);
    }
}
