<?php

namespace App\Http\Controllers\Shared;

use App\Http\Controllers\Controller;
use App\Http\Requests\CreateServerRequest;
use App\Http\Requests\UpdateServerRequest;
use App\Models\Server;
use App\Models\Game;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class ServerController extends Controller
{

    /**
     *  @OA\Get(
     *    tags={"Clients:Server"},
     *    path="/server",
     *    summary="Get server list",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="servers",
     *          type="array",
     *          @OA\Items(ref="#/components/schemas/Server")
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function index()
    {
        return response()->json([
            "servers" => Server::all()
        ]);
    }

    /**
     *  @OA\Post(
     *    tags={"Clients:Server"},
     *    path="/server",
     *    summary="Create a server",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/CreateServer"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="201",
     *      description="Resource created",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="server",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/Server"),
     *            @OA\Schema(@OA\Property(
     *              property="owner",
     *              ref="#/components/schemas/User"
     *            ))
     *          }
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function store(CreateServerRequest $request)
    {
        $data = $request->validated();
        $data['owner_id'] = Auth::user()->id;
        $data['password'] = Hash::make($data['password']);
        $server = Server::create($data);
        return $server->load('owner');
    }

    /**
     *  @OA\Get(
     *    tags={"Clients:Server"},
     *    path="/server/{id}",
     *    summary="Get a server",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="server",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/Server"),
     *            @OA\Schema(@OA\Property(
     *              property="owner",
     *              ref="#/components/schemas/User"
     *            ))
     *          }
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function show($id)
    {
        return response()->json([
            "server" => Server::findOrFail($id)->load('owner')
        ]);
    }

        /**
     *  @OA\Get(
     *    tags={"Clients:Server"},
     *    path="/server/{id}/game",
     *    summary="Get games from a server",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *         allOf={
     *           @OA\Schema(@OA\Property(
     *             property="games",
     *             ref="#/components/schemas/Game"
     *           ))
     *         }
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function index_games($id)
    {
        return response()->json([
            "games" => Server::findOrFail($id)->games
        ]);
    }

    /**
     *  @OA\Get(
     *    tags={"Clients:Server"},
     *    path="/server/{id}/game/{game_id}",
     *    summary="Get a game from a server",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Parameter(
     *      name="game_id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *         allOf={
     *           @OA\Schema(@OA\Property(
     *             property="game",
     *             ref="#/components/schemas/Game"
     *           ))
     *         }
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function show_games($id , $game_id)
    {
        return response()->json([
            "game" => Server::findOrFail($id)->games()->findOrFail($game_id)
        ]);
    }

    /**
     *  @OA\Put(
     *    tags={"Clients:Server"},
     *    path="/server/{id}",
     *    summary="Update a server",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/UpdateServer"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *     @OA\Response(
     *      response="403",
     *      ref="#/components/responses/403"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK"
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function update(UpdateServerRequest $request, $id)
    {
        $server = Server::findOrFail($id);
        if ($server->owner_id !== Auth::user()->id) {
            return response()->json(['message' => 'Not authorized.'], 403);
        }
        $data = $request->validated();
        if (array_key_exists('password', $data))
            $data['password'] = Hash::make($data['password']);
        $server->update($data);
        return response()->noContent();
    }

    /**
     *  @OA\Delete(
     *    tags={"Clients:Server"},
     *    path="/server/{id}",
     *    summary="Destroy a server",
     *    @OA\Parameter(
     *      name="id",
     *      in="path",
     *      required=true,
     *      @OA\Schema(type="integer")
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="403",
     *      ref="#/components/responses/403"
     *    ),
     *    @OA\Response(
     *      response="404",
     *      ref="#/components/responses/404"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK"
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function destroy($id)
    {
        $server = Server::findOrFail($id);
        $server->stop();
        if ($server->owner_id !== Auth::user()->id) {
            return response()->json(['message' => 'Not authorized.'], 403);
        }
        $server->delete();
        return response()->noContent();
    }
}
